package com.bsis.peuplement.service;

import com.bsis.peuplement.model.Dossier;

import com.bsis.peuplement.model.DossierDistance;
import com.bsis.peuplement.repository.DossierDistanceRepository;
import com.bsis.peuplement.repository.DossierRepository;
import com.bsis.peuplement.repository.RedevableRepository;
import com.bsis.peuplement.tasks.Tasks;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.URL;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;


@Service
public class DossierService {
    @Autowired
    private DossierRepository dossierRepository;
    //...................................................................
    //                              Read
    //...................................................................
    public List<Dossier> getAll()
    {
        return dossierRepository.findAll();
    }

    //................................
    public Optional<Dossier> getById(Long id)
    {
        return  dossierRepository.findById(id);
    }
    //...................................................................
    public long count() {
        return dossierRepository.count();
    }
    //...................................................................
    //                            Update
    //...................................................................
    public Dossier update(Dossier dossier){
        return	dossierRepository.save(dossier);
    }

 /*   private static final double SEUIL = 10.0 ;
    @Autowired
    private DossierRepository dossierRepository;

    @Autowired
    private RedevableRepository redevableRepository;

    @Autowired
    private RedevableService redevableService;

    @Autowired
    private DossierDistanceRepository dossierDistanceRepository;

    //String platFile= this.getClass().getClassLoader().getResource("platFile.csv")+"";
//    String platFile="C:/WorkSpace/Projects/Savoi/csv/platFile.csv";
//    String storAdr="C:/WorkSpace/Projects/Savoi/csv/storAdr.csv";
//    String log="C:/WorkSpace/Projects/Savoi/csv/log.csv";

    String platFile="./platFile.csv";
    String storAdr="./storAdr.csv";
    String log="./";
//    String log="C:/WorkSpace/Projects/Savoi/tmp/peuplement/";

//    String log ="";

//    {
//        try {
//            log = String.valueOf(new ClassPathResource("log.txt").getFile().toPath());
//            log = log.substring(0, log.length()-7)+"logs/";
//            System.out.println(log);
//        } catch (IOException e) {
//            throw new RuntimeException(e);
//        }
//    }


//    String storAdr= this.getClass().getClassLoader().getResource("storeAdr.csv")+"";
//    String log= this.getClass().getClassLoader().getResource("log")+"";


    //...................................................................
    //                              Create
    //...................................................................
    public Dossier add(Dossier dossier) {
        return dossierRepository.insert(dossier);
    }
    //...................................................................
    //                              Read
    //...................................................................
    public List<Dossier> getAll(){
        return dossierRepository.findAll();
    }
    //...................................................................
    public Optional<Dossier> getById(String id) {
        return dossierRepository.findById(id);
    }
    //...................................................................
    public Dossier getByDossierId(String dossierId) {
        return dossierRepository.findDossierByDossierId(dossierId);
    }
    //...................................................................
    public long count() {
        return dossierRepository.count();
    }
    //...................................................................
    //                            Update
    //...................................................................
    public Dossier update(Dossier dossier){
        return	dossierRepository.save(dossier);
    }
    //...................................................................
    //                          Delete
    //...................................................................
    public void deleteAll() {
        dossierRepository.deleteAll();
    }
    //...................................................................
    public void delete(String id) {
        dossierRepository.deleteById(id);
    }
    //...................................................................
    //                          RECHERCHE
    //...................................................................
    public String[] xyStorAdr(Dossier dossier) {

        String adresse = dossier.getAdresse();
        if (adresse == null) {adresse ="null";}

        String [] xy= Tasks.crdnXYLambert93StorAdr(adresse,storAdr);
        return xy;
    }
    //...................................................................
    // Distance Between p1(X1 Y1) & p2(X2 Y2)
    //...................................................................
    public double distance(String x1, String y1, String x2, String y2 )
    {
        return Tasks.calculateDistance(Double.parseDouble(x1),Double.parseDouble(y1),Double.parseDouble(x2),Double.parseDouble(y2));
    }
    //.................................................................................
    // Peuplement one dossier --> Update Dossier(dossierId) DossierDistance(dossierId)
    //..................................................................................
    public DossierDistance peuplementById (String dossierId) {
        Dossier dossier = getByDossierId(dossierId);
        DossierDistance dossierDistance = new DossierDistance();
        dossierDistance.setDossierId(dossierId);
        dossierDistance.setRedevableId(dossier.getRedevableId());
        dossierDistance.setSiretRedevable(redevableService.getByRedevableId(dossier.getRedevableId()).getCodeSiret());
        if (dossier.getAdresse() == null)
        {
            dossierDistance.setDossierState("Err Adresse dossier");
            dossierDistanceRepository.save(dossierDistance);
        }
        else
        {
           dossierDistance.setAdresse(dossier.getAdresse());
           String[] point1 = xyStorAdr(dossier);
           if (point1[0] == "-111111111111111")
           {
               dossierDistance.setDossierState("Err xyStorAdr not found");
           }
           else
           {
              dossierDistance.setxStorAdr(point1[0]);
              dossierDistance.setyStorAdr(point1[1]);
              String siret = "---";
              String[] point2 = redevableService.xyFichierPlat(dossier, siret);
              if (siret == "")
              {
                  dossierDistance.setDossierState("Err siretRedevable not found");
              }
              else
              {
                  if (siret == "00000000000000")
                  {
                      dossierDistance.setDossierState("Err siret null");
                  }
                  else
                  {
                      if (point2[0] == "-99999999999999999")
                      {
                         dossierDistance.setDossierState("Err xyFlatFile not found");
                      }
                      else
                      {
                          dossierDistance.setDossierState("OK");
                          dossierDistance.setSiretDossier(dossierDistance.getSiretRedevable());
                          dossierDistance.setxFlatFile(point2[0]);
                          dossierDistance.setyFlatFile(point2[1]);
                          dossierDistance.setDistance("" + distance(point1[0], point1[1], point2[0], point2[1]));
                          dossier.setSiret(dossierDistance.getSiretRedevable());
                      }
                  }
              }
           }
           dossierDistance.setDossierId(dossier.getDossierId());
           dossierDistanceRepository.save(dossierDistance);
           dossierRepository.save(dossier);
          // System.out.println(dossierDistance);
        }
        return dossierDistance;
    }
    //........................................................
    // All distance --> create of dossierDistance list
    //........................................................
    public List<DossierDistance> peuplementAll()
    {
        List<DossierDistance> dossierDistances = new ArrayList<>();
        // create a new logFile
        LocalDateTime currentDateTime = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd_HHmmss");
        String formattedDateTime = currentDateTime.format(formatter);

        String logFile=log+formattedDateTime+".csv";
        List<Dossier> dossiers = getAll();  // recuperer tous les dossier
        Long cpt = 1L;
        Long total = dossierRepository.count();

        //*********************************************************************************************
        try {
            System.out.println(logFile);
            FileWriter fileWriter = new FileWriter(logFile, false);
            fileWriter.write("");
            writeToFile(logFile,"dossierState;dossierId;adresse;siretDossier;xStorAdr;yStorAdr;redevableId;siretRedevable;xFlatFile;yFlatFile;distance\n");
            for (Dossier dossier:dossiers )
            {
                DossierDistance dossierDistance = peuplementById(dossier.getDossierId());
                writeToFile(logFile,dossierDistance.getDossierState()+";"+dossierDistance.getDossierId()+";"+dossierDistance.getAdresse()+";"+dossierDistance.getSiretDossier()+";"+dossierDistance.getxStorAdr()+";"+dossierDistance.getyStorAdr()+";"+dossierDistance.getRedevableId()+";"+dossierDistance.getSiretRedevable()+";"+dossierDistance.getxFlatFile()+";"+dossierDistance.getyFlatFile()+";"+dossierDistance.getDistance()+"\n");
//              writeToFile(logFile,"-;-;-;-;-;-;-;-;-;-;-\n");
                System.out.println(dossierDistance);
                System.out.println("["+cpt+"/"+total+" dossiers]");

                cpt++;
            }
        } catch (IOException e) {
            System.out.println("Error while writting log file : " + logFile);
            e.printStackTrace();
        }
        return dossierDistances;
    }
   //..........................................................
    public static void writeToFile(String logFile, String content) throws IOException {
        try (FileWriter fileWriter = new FileWriter(logFile, true)) {
            fileWriter.write(content);
        }
    }
    //............................................................
    public List<DossierDistance>  peuplementSet(String set)
    {
        String path="/"+set;
        List<DossierDistance> dossierDistances = new ArrayList<>();
        List<String> dossierIds = new ArrayList<>();
        try (BufferedReader reader = new BufferedReader(new FileReader(path)))
        {
            String ligne;
            while ((ligne = reader.readLine()) != null)
            {
                String[] valeurs = ligne.split(";");
                for (String valeur: valeurs)
                {
                    dossierIds.add(valeur);
                }
            }
        }
        catch (IOException e)
        {
            System.out.println("File not found");
            e.printStackTrace();
        }
        for (String dossierId:dossierIds)
        {
            DossierDistance dossierDistance=peuplementById(dossierId);
            dossierDistances.add(dossierDistance);
            dossierDistanceRepository.save(dossierDistance);
        }
        return dossierDistances;
    }

    // ............................................................
    public String init ()
    {

//        String stringUrl ="http://10.18.126.211:22/";
//        try {
//            URL url = new URL(stringUrl);
//
//            // Obtention du chemin du fichier à partir de l'URL
//            String filePath = url.getPath()+"/s25_savoi_jboss/logs/sav.txt";
//            File file = new File(filePath);
//            //***********************
//            try (BufferedReader reader = new BufferedReader(new FileReader(filePath))) {
//                String line;
//
//                // Lecture ligne par ligne et affichage
//                while ((line = reader.readLine()) != null) {
//                    System.out.println(line);
//                }
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//
//            //************************
//
//            System.out.println("Chemin du fichier : " + file.getAbsolutePath());
//        } catch (Exception e) {
//            e.printStackTrace();
//        }




        List<Dossier> dossiers = dossierRepository.findAll();
        for (Dossier dossier: dossiers)
        {
            dossier.setSiret("");
            dossierRepository.save(dossier);
        }
        dossierDistanceRepository.deleteAll();
        return "Init process completed successfully";
    }

    public String FlatFileFiltrer() {
       String inputFile= String.valueOf(getClass().getClassLoader().getResource("GeolocalisationEtablissement_Sirene_pour_etudes_statistiques_utf8.csv"));
       String outputFile= String.valueOf(getClass().getClassLoader().getResource("FilteredFile1.csv"));
       Tasks.filtred(inputFile, outputFile);
        return "OK";
    }*/
}

